//
//  NTBox.m
//  NewtonTest
//
//  Created by William Thimbleby on Sat Mar 05 2005.
//  Copyright (c) 2005 __MyCompanyName__. All rights reserved.
//

#import "NTBox.h"
#import <OpenGL/gl.h>

GLuint GetOpenGLTextureFromPath(NSString *path) 
{ 
	NSBitmapImageRep	*bitmapRep; 
	int					samples, bits, width, height; 
	BOOL				alpha; 
	GLuint				texName; 
	GLenum				type, format; 
 	
	bitmapRep = [NSBitmapImageRep imageRepWithContentsOfFile:path]; 
 	
	if([bitmapRep isPlanar]) 
	{ 
		NSLog(@"GetOpenGLTexture: Can't handle planar images."); 
		return -1; 
	} 
 	
	// get image data 
	samples = [bitmapRep samplesPerPixel]; 
	bits = [bitmapRep bitsPerPixel]; 
	alpha = [bitmapRep hasAlpha]; 
	width = [bitmapRep pixelsWide]; 
	height = [bitmapRep pixelsHigh]; 

	if(alpha) 
	{ 
		samples --; 
		bits *= 0.75; 
	} 
 	
	switch(bits/samples) 
	{ 
		case 8: 
			type = GL_UNSIGNED_BYTE; 
			break; 
		case 16: 
			type = GL_UNSIGNED_SHORT; 
			break; 
		case 32: 
			type = GL_UNSIGNED_INT; 
			break; 
		default: 
			NSLog(@"GetOpenGLTexture: Can't handle bits per sample of %d/%d.", bits, samples); 
			return -1; 
	} 
 	
	switch(samples) 
	{ 
		case 3: 
			format = alpha? GL_RGBA : GL_RGB; 
			break; 
		case 1: 
			format = alpha? GL_LUMINANCE_ALPHA : GL_LUMINANCE; 
			break; 
		default: 
			NSLog(@"GetOpenGLTexture sample confused: %d, %d, %d, %d, %d", samples, bits, alpha, width, height); 
			return -1; 
	} 
 	

	//generate the texture 
	glGenTextures(1, &texName); 
	glBindTexture(GL_TEXTURE_2D, texName);
	
	
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	
	
	glTexImage2D(GL_TEXTURE_2D, 0, format, width, height, 0, format, type, [bitmapRep bitmapData]); 
 	
	return texName; 
}

void transformPosition(const float *pos, const float *R)
{
	GLfloat matrix[16];
	matrix[0]=R[0];
	matrix[1]=R[4];
	matrix[2]=R[8];
	matrix[3]=0;
	matrix[4]=R[1];
	matrix[5]=R[5];
	matrix[6]=R[9];
	matrix[7]=0;
	matrix[8]=R[2];
	matrix[9]=R[6];
	matrix[10]=R[10];
	matrix[11]=0;
	matrix[12]=pos[0];
	matrix[13]=pos[1];
	matrix[14]=pos[2];
	matrix[15]=1;
	
	glPushMatrix();
	glMultMatrixf (matrix);
}

GLuint woodTex = 0;
GLuint boxList = 0;

@implementation NTBox

- (id)initWithWorld:(dWorldID)world inSpace:(dSpaceID)space atX:(float)x atY:(float)y atZ:(float)z
{
	self = [super init];
	
	if(woodTex == 0)
		woodTex = GetOpenGLTextureFromPath([NSString stringWithFormat:@"%@/wood.png", [[NSBundle mainBundle] resourcePath]]);
	if(boxList == 0)
		[self generateList];
	
	dMass m;
	
	body = dBodyCreate (world);
	dBodySetPosition (body, x, y, z);
	//dBodySetData (body, (void*) i);
	dMassSetBox (&m, 8, 0.5, 0.5, 0.5);
	geom = dCreateBox (space, 0.5, 0.5, 0.5);
	
	dGeomSetBody (geom, body);
    dBodySetMass (body, &m);

	return self;
}

- (void)generateList
{
	boxList = glGenLists(1);
	glNewList(boxList, GL_COMPILE);
	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, woodTex);
	
	glBegin(GL_QUADS);
	// front
	glNormal3f(0.0, 0.0, 1.0);
	glTexCoord2f(0.0, 0.0); glVertex3f(-1, -1, 1);
	glTexCoord2f(1.0, 0.0); glVertex3f( 1, -1, 1);
	glTexCoord2f(1.0, 1.0); glVertex3f( 1,  1, 1);
	glTexCoord2f(0.0, 1.0); glVertex3f(-1,  1, 1);
	
	// back
	glNormal3f(0.0, 0.0, -1.0);
	glTexCoord2f(0.0, 0.0); glVertex3f( 1, -1, -1);
	glTexCoord2f(1.0, 0.0); glVertex3f(-1, -1, -1);
	glTexCoord2f(1.0, 1.0); glVertex3f(-1,  1, -1);
	glTexCoord2f(0.0, 1.0); glVertex3f( 1,  1, -1);
	
	// top
	glNormal3f(0.0, 1.0, 0.0);
	glTexCoord2f(0.0, 0.0); glVertex3f(-1,  1,  1);
	glTexCoord2f(1.0, 0.0); glVertex3f( 1,  1,  1);
	glTexCoord2f(1.0, 1.0); glVertex3f( 1,  1, -1);
	glTexCoord2f(0.0, 1.0); glVertex3f(-1,  1, -1);
	
	// bottom
	glNormal3f(0.0, -1.0, 0.0);
	glTexCoord2f(0.0, 0.0); glVertex3f(-1, -1, -1);
	glTexCoord2f(1.0, 0.0); glVertex3f( 1, -1, -1);
	glTexCoord2f(1.0, 1.0); glVertex3f( 1, -1,  1);
	glTexCoord2f(0.0, 1.0); glVertex3f(-1, -1,  1);
	
	// left
	glNormal3f(-1.0, 0.0, 0.0);
	glTexCoord2f(0.0, 0.0); glVertex3f(-1, -1, -1);
	glTexCoord2f(1.0, 0.0); glVertex3f(-1, -1,  1);
	glTexCoord2f(1.0, 1.0); glVertex3f(-1,  1,  1);
	glTexCoord2f(0.0, 1.0); glVertex3f(-1,  1, -1);
	
	// right
	glNormal3f(1.0, 0.0, 0.0);
	glTexCoord2f(0.0, 0.0); glVertex3f(1,  -1,  1);
	glTexCoord2f(1.0, 0.0); glVertex3f(1,  -1, -1);
	glTexCoord2f(1.0, 1.0); glVertex3f(1,   1, -1);
	glTexCoord2f(0.0, 1.0); glVertex3f(1,   1,  1);
	
	glEnd();
	
	glDisable(GL_TEXTURE_2D);
	
	glEndList();
}

- (void)draw
{
	transformPosition(dBodyGetPosition(body), dBodyGetRotation(body));
	glScalef(0.25,0.25,0.25);
	
	glCallList(boxList);
	
	glPopMatrix();
}

@end
